
/* Generated by Interface Builder */

#import "ParamControl.h"
#import <appkit/Form.h>
#import <appkit/TextField.h>
#import <appkit/Matrix.h>
#import <stdlib.h>


#import <math.h>

@implementation ParamControl

- init
{
	int i;
		/* set default parameters */
	dt=0.10;
	ndata=512;
	incAngle=0.0;
	type=P;
	sourceTime=NULL;
	
	maxSourceLength=(int)2.0/dt;
	sourceTime=(float *)calloc(maxSourceLength,sizeof(float));
	for(i=0;i<maxSourceLength;i++);
		sourceTime[i]=0.0;
	sourceTime[0]=1.0;

	
	return self;
}

- prepareArea
{

		/* draw incident view */
	[IncidentPlot display];
	[SourcePlot display];
	
		/* initialize Forms */
	[SampleForm setFloatValue:dt at:0];
	[IncidentForm setFloatValue:incAngle at:0];
	[SourceForm setFloatValue:maxSourceLength*dt at:0];
	[NPointsForm setIntValue:ndata at:0];
	[TypeMatrix selectCellAt:type :0];
	
		/* initialize the component view in the parameter controller also */
	if(delegate && [delegate respondsTo:@selector(disableComp::)])
		[delegate disableComp:self:1];
	if(delegate && [delegate respondsTo:@selector(enableComp::)])
		[delegate enableComp:self:0];

	
	return self;
}

- sendParameters:(float *)sampleint:(float *)angle:(int *)num:(int *)itype:(float **)sTime:(int *)lSTime;
{
	*sampleint=dt;
	*angle=incAngle;
	*num=ndata;
	*itype=type;
	*sTime=sourceTime;
	*lSTime=maxSourceLength;
	
	return self;
}

		/* action methods */
- setSampleInterval:sender
{
	dt=[sender floatValueAt:0];
	
		/* release SourceTime array */
	[self setSourceLength:self];
	
	return self;
}

- setSourceLength:sender
{
	int i;
	maxSourceLength=(int)[SourceForm floatValueAt:0]/dt;
	if(sourceTime != NULL)
	{
		free((void *)sourceTime);
		sourceTime=(float *)calloc(maxSourceLength,sizeof(float));
		for(i=0;i<maxSourceLength;i++)
			sourceTime[i]=0.0;
		sourceTime[0]=1.0;
		
		[SourcePlot display];
		
	}
	return self;
}

- setNPoints:sender
{
	int i;
	
	ndata=[sender intValueAt:0];
	for(i=0;i<15;i++)
	{
		if(ndata == (int)pow(2.0,(float)i))
			break;
	}
	if(i == 15)	/* input time series length is not a power of two */
	{
		for(i=0;ndata > (int)pow(2.0,(float)i);i++);	/* choose nearest power of two */
		ndata=(int)pow(2.0,(float)(i-1));
		[sender setIntValue:ndata at:0];
	}
				
	return self;
}

- setIncidentAngle:sender
{
	incAngle=[sender floatValueAt:0];
	
		/* redisplay incident plot */
	[IncidentPlot display];
	
		/* fix the component matrix in the delegated view */
	if(type == P)
	{
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:1];
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:0];
	}
	if(type == SV)
	{
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:0];
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:1];
	}	

		/* if incAngle == 0 and the wave type is P or SV one component will not exist */
	if(incAngle == 0)
	{
		if(type == P)
		{
			if(delegate && [delegate respondsTo:@selector(disableComp::)])
				[delegate disableComp:self:1];
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:0];
		}
		if(type == SV)
		{
			if(delegate && [delegate respondsTo:@selector(disableComp::)])
				[delegate disableComp:self:0];
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:1];
		}
	}
	
	return self;
}

- setIncidentWaveType:sender
{
	type = [TypeMatrix selectedRow];
	
		/* set acceptable component types based on incident wave */
	if(type == SH)	/* Sh wave */
	{
		if(delegate && [delegate respondsTo:@selector(disableComp::)])
			[delegate disableComp:self:0];
		if(delegate && [delegate respondsTo:@selector(disableComp::)])
			[delegate disableComp:self:1];
		if(delegate && [delegate respondsTo:@selector(enableComp::)])
			[delegate enableComp:self:2];			
	}
	if(type == SV)	/* Sv wave */
	{
		if(delegate && [delegate respondsTo:@selector(disableComp::)])
			[delegate disableComp:self:2];
		if(incAngle == 0.0)
		{
			if(delegate && [delegate respondsTo:@selector(disableComp::)])
				[delegate disableComp:self:0];
		}
		else
		{
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:0];
		}
		if(delegate && [delegate respondsTo:@selector(enableComp::)])
			[delegate enableComp:self:1];

	}
	if(type == P)	/* P wave */
	{
		if(delegate && [delegate respondsTo:@selector(disableComp::)])
			[delegate disableComp:self:2];
		if(incAngle == 0.0)
		{
			if(delegate && [delegate respondsTo:@selector(disableComp::)])
				[delegate disableComp:self:1];
		}
		else
		{
			if(delegate && [delegate respondsTo:@selector(enableComp::)])
				[delegate enableComp:self:1];
		}
		if(delegate && [delegate respondsTo:@selector(enableComp::)])
			[delegate enableComp:self:0];			
	}

    	return self;
}

	/* delegated methods */
- (float)getIncidentAngle
{
	return incAngle;
}

- putIncidentAngle:(float)angle
{
	incAngle=angle;
	
		/* update the incident angle form */
	[IncidentForm setFloatValue:incAngle at:0];
	
		/* update the viable set of components */
	[self setIncidentWaveType:self];
	
	return self;
}

- dataChanged:sender
{
	[self prepareArea];
	
	return self;
}

- sendDataToPlot:sender:(float **)x:(int *)num:(float *)sample
{	
	*num=maxSourceLength;
	*sample=dt;
	*x=&sourceTime[0];

	return self;
}

@end
